@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #include <termios.h>
@@ -21,6 +35,7 @@
 
 #include "user.h"
 #include "fatal_assert.h"
+#include "pty_compat.h"
 #include "networktransport.cc"
 #include "select.h"
 
@@ -59,10 +74,14 @@ int main( int argc, char *argv[] )
 
   if ( server ) {
     Select &sel = Select::get_instance();
-    sel.add_fd( n->fd() );
     uint64_t last_num = n->get_remote_state_num();
     while ( true ) {
       try {
+	sel.clear_fds();
+	std::vector< int > fd_list( n->fds() );
+	assert( fd_list.size() == 1 ); /* servers don't hop */
+	int network_fd = fd_list.back();
+	sel.add_fd( network_fd );
 	if ( sel.select( n->wait_time() ) < 0 ) {
 	  perror( "select" );
 	  exit( 1 );
@@ -70,7 +89,7 @@ int main( int argc, char *argv[] )
 	
 	n->tick();
 
-	if ( sel.read( n->fd() ) ) {
+	if ( sel.read( network_fd ) ) {
 	  n->recv();
 
 	  if ( n->get_remote_state_num() != last_num ) {
@@ -101,10 +120,18 @@ int main( int argc, char *argv[] )
     }
 
     Select &sel = Select::get_instance();
-    sel.add_fd( STDIN_FILENO );
-    sel.add_fd( n->fd() );
 
     while( true ) {
+      sel.clear_fds();
+      sel.add_fd( STDIN_FILENO );
+
+      std::vector< int > fd_list( n->fds() );
+      for ( std::vector< int >::const_iterator it = fd_list.begin();
+	    it != fd_list.end();
+	    it++ ) {
+	sel.add_fd( *it );
+      }
+
       try {
 	if ( sel.select( n->wait_time() ) < 0 ) {
 	  perror( "select" );
@@ -118,7 +145,22 @@ int main( int argc, char *argv[] )
 	  n->get_current_state().push_back( Parser::UserByte( x ) );
 	}
 
-	if ( sel.read( n->fd() ) ) {
+	bool network_ready_to_read = false;
+	for ( std::vector< int >::const_iterator it = fd_list.begin();
+	      it != fd_list.end();
+	      it++ ) {
+	  if ( sel.read( *it ) ) {
+	    /* packet received from the network */
+	    /* we only read one socket each run */
+	    network_ready_to_read = true;
+	  }
+
+	  if ( sel.error( *it ) ) {
+	    break;
+	  }
+	}
+
+	if ( network_ready_to_read ) {
 	  n->recv();
 	}
       } catch ( NetworkException e ) {
