@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #include "config.h"
@@ -23,6 +37,7 @@
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/ioctl.h>
@@ -40,17 +55,14 @@
 #include <time.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_UTMPX_H
+#include <utmpx.h>
+#endif
+
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif
 
-#include "completeterminal.h"
-#include "swrite.h"
-#include "user.h"
-#include "fatal_assert.h"
-#include "locale_utils.h"
-#include "select.h"
-
 #if HAVE_PTY_H
 #include <pty.h>
 #elif HAVE_UTIL_H
@@ -61,6 +73,15 @@
 #include <libutil.h>
 #endif
 
+#include "completeterminal.h"
+#include "swrite.h"
+#include "user.h"
+#include "fatal_assert.h"
+#include "locale_utils.h"
+#include "pty_compat.h"
+#include "select.h"
+#include "timestamp.h"
+
 #ifndef _PATH_BSHELL
 #define _PATH_BSHELL "/bin/sh"
 #endif
@@ -87,6 +108,7 @@ void print_usage( const char *argv0 )
 void print_motd( void );
 void chdir_homedir( void );
 bool motd_hushed( void );
+void warn_unattached( const string & ignore_entry );
 
 /* Simple spinloop */
 void spin( void )
@@ -99,19 +121,36 @@ void spin( void )
     req.tv_sec = 0;
     req.tv_nsec = 100000000; /* 0.1 sec */
     nanosleep( &req, NULL );
+    freeze_timestamp();
   }
 }
 
 string get_SSH_IP( void )
 {
   const char *SSH_CONNECTION = getenv( "SSH_CONNECTION" );
-  fatal_assert( SSH_CONNECTION );
+  if ( !SSH_CONNECTION ) { /* Older sshds don't set this */
+    fprintf( stderr, "Warning: SSH_CONNECTION not found; binding to any interface.\n" );
+    return string( "0.0.0.0" );
+  }
   char *SSH_writable = strdup( SSH_CONNECTION );
   fatal_assert( SSH_writable );
+
   strtok( SSH_writable, " " );
   strtok( NULL, " " );
   const char *local_interface_IP = strtok( NULL, " " );
-  fatal_assert( local_interface_IP );
+  if ( !local_interface_IP ) {
+    fprintf( stderr, "Warning: Could not parse SSH_CONNECTION; binding to any interface.\n" );
+    return string( "0.0.0.0" );
+  }
+
+  /* Strip IPv6 prefix. */
+  const char IPv6_prefix[] = "::ffff:";
+
+  if ( ( strlen( local_interface_IP ) > strlen( IPv6_prefix ) )
+       && ( 0 == strncasecmp( local_interface_IP, IPv6_prefix, strlen( IPv6_prefix ) ) ) ) {
+    return string( local_interface_IP + strlen( IPv6_prefix ) );
+  }
+
   return string( local_interface_IP );
 }
 
@@ -294,7 +333,7 @@ int run_server( const char *desired_ip, const char *desired_port,
   struct winsize window_size;
   if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &window_size ) < 0 ) {
     perror( "ioctl TIOCGWINSZ" );
-    fprintf( stderr, "If running with ssh, please use -t argument to provide a PTY.\n" );
+    fprintf( stderr, "If running with ssh, please use ssh -t to provide a PTY.\n" );
     exit( 1 );
   }
 
@@ -336,10 +375,6 @@ int run_server( const char *desired_ip, const char *desired_port,
     _exit( 0 );
   }
 
-  if ( setsid() < 0 ) {
-    perror( "setsid" );
-  }
-
   fprintf( stderr, "\nmosh-server (%s)\n", PACKAGE_STRING );
   fprintf( stderr, "Copyright 2012 Keith Winstein <mosh-devel@mit.edu>\n" );
   fprintf( stderr, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.\n\n" );
@@ -361,11 +396,25 @@ int run_server( const char *desired_ip, const char *desired_port,
   /* close file descriptors */
   if ( !verbose ) {
     /* Necessary to properly detach on old versions of sshd (e.g. RHEL/CentOS 5.0). */
-    fclose( stdin );
-    fclose( stdout );
-    fclose( stderr );
+    int nullfd;
+
+    nullfd = open( "/dev/null", O_RDWR );
+    if ( nullfd == -1 ) {
+      perror( "dup2" );
+      exit( 1 );
+    }
+
+    if ( dup2 ( nullfd, STDIN_FILENO ) < 0 ||
+         dup2 ( nullfd, STDOUT_FILENO ) < 0 ||
+         dup2 ( nullfd, STDERR_FILENO ) < 0 ) {
+      perror( "dup2" );
+      exit( 1 );
+    }
   }
 
+  char utmp_entry[ 64 ] = { 0 };
+  snprintf( utmp_entry, 64, "mosh [%d]", getpid() );
+
   /* Fork child process */
   pid_t child = forkpty( &master, NULL, &child_termios, &window_size );
 
@@ -377,11 +426,6 @@ int run_server( const char *desired_ip, const char *desired_port,
   if ( child == 0 ) {
     /* child */
 
-    /* reopen stdio */
-    stdin = fdopen( STDIN_FILENO, "r" );
-    stdout = fdopen( STDOUT_FILENO, "w" );
-    stderr = fdopen( STDERR_FILENO, "w" );
-
     /* reenable signals */
     struct sigaction sa;
     sa.sa_handler = SIG_DFL;
@@ -418,6 +462,7 @@ int run_server( const char *desired_ip, const char *desired_port,
 
     if ( with_motd && (!motd_hushed()) ) {
       print_motd();
+      warn_unattached( utmp_entry );
     }
 
     Crypto::reenable_dumping_core();
@@ -429,12 +474,10 @@ int run_server( const char *desired_ip, const char *desired_port,
   } else {
     /* parent */
 
-    #ifdef HAVE_UTEMPTER
+#ifdef HAVE_UTEMPTER
     /* make utmp entry */
-    char tmp[ 64 ];
-    snprintf( tmp, 64, "mosh [%d]", getpid() );
-    utempter_add_record( master, tmp );
-    #endif
+    utempter_add_record( master, utmp_entry );
+#endif
 
     try {
       serve( master, terminal, *network );
@@ -467,8 +510,6 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 {
   /* prepare to poll for events */
   Select &sel = Select::get_instance();
-  sel.add_fd( network.fd() );
-  sel.add_fd( host_fd );
   sel.add_signal( SIGTERM );
   sel.add_signal( SIGINT );
 
@@ -487,10 +528,18 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 
       const int timeout_if_no_client = 60000;
       int timeout = min( network.wait_time(), terminal.wait_time( now ) );
-      if ( !network.has_remote_addr() ) {
+      if ( !network.get_remote_state_num() ) {
         timeout = min( timeout, timeout_if_no_client );
       }
 
+      /* poll for events */
+      sel.clear_fds();
+      std::vector< int > fd_list( network.fds() );
+      assert( fd_list.size() == 1 ); /* servers don't hop */
+      int network_fd = fd_list.back();
+      sel.add_fd( network_fd );
+      sel.add_fd( host_fd );
+
       int active_fds = sel.select( timeout );
       if ( active_fds < 0 ) {
 	perror( "select" );
@@ -500,7 +549,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
       now = Network::timestamp();
       uint64_t time_since_remote_state = now - network.get_latest_remote_state().timestamp;
 
-      if ( sel.read( network.fd() ) ) {
+      if ( sel.read( network_fd ) ) {
 	/* packet received from the network */
 	network.recv();
 	
@@ -606,7 +655,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	}
       }
       
-      if ( sel.error( network.fd() ) ) {
+      if ( sel.error( network_fd ) ) {
 	/* network problem */
 	break;
       }
@@ -657,7 +706,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	}
       }
 
-      if ( !network.has_remote_addr()
+      if ( !network.get_remote_state_num()
            && time_since_remote_state >= uint64_t( timeout_if_no_client ) ) {
         fprintf( stderr, "No connection within %d seconds.\n",
                  timeout_if_no_client / 1000 );
@@ -708,7 +757,7 @@ void chdir_homedir( void )
   struct passwd *pw = getpwuid( geteuid() );
   if ( pw == NULL ) {
     perror( "getpwuid" );
-    /* non-fatal */
+    return; /* non-fatal */
   }
 
   if ( chdir( pw->pw_dir ) < 0 ) {
@@ -726,3 +775,65 @@ bool motd_hushed( void )
   struct stat buf;
   return (0 == lstat( ".hushlogin", &buf ));
 }
+
+string mosh_read_line( FILE *file )
+{
+  string ret;
+  while ( !feof( file ) ) {
+    char next = getc( file );
+    if ( next == '\n' ) {
+      return ret;
+    }
+    ret.push_back( next );
+  }
+  return ret;
+}
+
+void warn_unattached( const string & ignore_entry )
+{
+#ifdef HAVE_UTMPX_H
+  /* get username */
+  const struct passwd *pw = getpwuid( geteuid() );
+  if ( pw == NULL ) {
+    perror( "getpwuid" );
+    /* non-fatal */
+    return;
+  }
+
+  const string username( pw->pw_name );
+
+  /* look for unattached sessions */
+  vector< string > unattached_mosh_servers;
+
+  while ( struct utmpx *entry = getutxent() ) {
+    if ( (entry->ut_type == USER_PROCESS)
+	 && (username == string( entry->ut_user )) ) {
+      /* does line show unattached mosh session */
+      string text( entry->ut_host );
+      if ( (text.substr( 0, 5 ) == "mosh ")
+	   && (text != ignore_entry) ) {
+	unattached_mosh_servers.push_back( text );
+      }
+    }
+  }
+
+  /* print out warning if necessary */
+  if ( unattached_mosh_servers.empty() ) {
+    return;
+  } else if ( unattached_mosh_servers.size() == 1 ) {
+    printf( "\033[37;44mMosh: You have a detached Mosh session on this server (%s).\033[m\n\n",
+	    unattached_mosh_servers.front().c_str() );
+  } else {
+    string pid_string;
+
+    for ( vector< string >::const_iterator it = unattached_mosh_servers.begin();
+	  it != unattached_mosh_servers.end();
+	  it++ ) {
+      pid_string += "        - " + *it + "\n";
+    }
+
+    printf( "\033[37;44mMosh: You have %d detached Mosh sessions on this server, with PIDs:\n%s\033[m\n",
+	    (int)unattached_mosh_servers.size(), pid_string.c_str() );
+  }
+#endif /* HAVE_UTMPX_H */
+}
