@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #include "config.h"
@@ -42,7 +56,9 @@
 #include "user.h"
 #include "fatal_assert.h"
 #include "locale_utils.h"
+#include "pty_compat.h"
 #include "select.h"
+#include "timestamp.h"
 
 #include "networktransport.cc"
 
@@ -293,8 +309,6 @@ void STMClient::main( void )
 
   /* prepare to poll for events */
   Select &sel = Select::get_instance();
-  sel.add_fd( network->fd() );
-  sel.add_fd( STDIN_FILENO );
 
   while ( 1 ) {
     try {
@@ -307,14 +321,41 @@ void STMClient::main( void )
 	wait_time = min( 250, wait_time );
       }
 
+      /* poll for events */
+      /* network->fd() can in theory change over time */
+      sel.clear_fds();
+      std::vector< int > fd_list( network->fds() );
+      for ( std::vector< int >::const_iterator it = fd_list.begin();
+	    it != fd_list.end();
+	    it++ ) {
+	sel.add_fd( *it );
+      }
+      sel.add_fd( STDIN_FILENO );
+
       int active_fds = sel.select( wait_time );
       if ( active_fds < 0 ) {
 	perror( "select" );
 	break;
       }
 
-      if ( sel.read( network->fd() ) ) {
-	/* packet received from the network */
+      bool network_ready_to_read = false;
+
+      for ( std::vector< int >::const_iterator it = fd_list.begin();
+	    it != fd_list.end();
+	    it++ ) {
+	if ( sel.read( *it ) ) {
+	  /* packet received from the network */
+	  /* we only read one socket each run */
+	  network_ready_to_read = true;
+	}
+
+	if ( sel.error( *it ) ) {
+	  /* network problem */
+	  break;
+	}
+      }
+
+      if ( network_ready_to_read ) {
 	if ( !process_network_input() ) { return; }
       }
     
@@ -350,11 +391,6 @@ void STMClient::main( void )
         }
       }
 
-      if ( sel.error( network->fd() ) ) {
-	/* network problem */
-	break;
-      }
-
       if ( sel.error( STDIN_FILENO ) ) {
 	/* user problem */
 	if ( !network->has_remote_addr() ) {
@@ -417,6 +453,7 @@ void STMClient::main( void )
       req.tv_sec = 0;
       req.tv_nsec = 200000000; /* 0.2 sec */
       nanosleep( &req, NULL );
+      freeze_timestamp();
     } catch ( Crypto::CryptoException e ) {
       if ( e.fatal ) {
         throw;
