@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #include <algorithm>
@@ -520,6 +534,16 @@ void PredictionEngine::cull( const Framebuffer &fb )
 	  }
 	}
 
+	/* match rest of row to the actual renditions */
+	{
+	  const Renditions &actual_renditions = fb.get_cell( i->row_num, j->col )->renditions;
+	  for ( overlay_cells_type::iterator k = j;
+		k != i->overlay_cells.end();
+		k++ ) {
+	    k->replacement.renditions = actual_renditions;
+	  }
+	}
+
 	/* no break */
       case CorrectNoCredit:
 	j->reset();
@@ -728,6 +752,18 @@ void PredictionEngine::new_user_byte( char the_byte, const Framebuffer &fb )
 	cell.tentative_until_epoch = prediction_epoch;
 	cell.expire( local_frame_sent + 1, now );
 	cell.replacement.renditions = fb.ds.get_renditions();
+
+	/* heuristic: match renditions of character to the left */
+	if ( cursor().col > 0 ) {
+	  ConditionalOverlayCell &prev_cell = the_row.overlay_cells[ cursor().col - 1 ];
+	  const Cell *prev_cell_actual = fb.get_cell( cursor().row, cursor().col - 1 );
+	  if ( prev_cell.active && (!prev_cell.unknown) ) {
+	    cell.replacement.renditions = prev_cell.replacement.renditions;
+	  } else {
+	    cell.replacement.renditions = prev_cell_actual->renditions;
+	  }
+	}
+
 	cell.replacement.contents.clear();
 	cell.replacement.contents.push_back( ch );
 	cell.original_contents.push_back( *fb.get_cell( cursor().row, cursor().col ) );
