@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #include "config.h"
@@ -26,19 +40,13 @@
 #include <errno.h>
 #include <unistd.h>
 
-#if HAVE_CLOCK_GETTIME
- #include <time.h>
-#elif HAVE_MACH_ABSOLUTE_TIME
- #include <mach/mach_time.h>
-#elif HAVE_GETTIMEOFDAY
- #include <sys/time.h>
-#endif
-
 #include "dos_assert.h"
 #include "byteorder.h"
 #include "network.h"
 #include "crypto.h"
 
+#include "timestamp.h"
+
 using namespace std;
 using namespace Network;
 using namespace Crypto;
@@ -73,7 +81,8 @@ string Packet::tostring( Session *session )
 {
   uint64_t direction_seq = (uint64_t( direction == TO_CLIENT ) << 63) | (seq & SEQUENCE_MASK);
 
-  uint16_t ts_net[ 2 ] = { htobe16( timestamp ), htobe16( timestamp_reply ) };
+  uint16_t ts_net[ 2 ] = { static_cast<uint16_t>( htobe16( timestamp ) ),
+                           static_cast<uint16_t>( htobe16( timestamp_reply ) ) };
 
   string timestamps = string( (char *)ts_net, 2 * sizeof( uint16_t ) );
 
@@ -98,11 +107,43 @@ Packet Connection::new_packet( string &s_payload )
   return p;
 }
 
-void Connection::setup( void )
+void Connection::hop_port( void )
 {
-  /* create socket */
-  sock = socket( AF_INET, SOCK_DGRAM, 0 );
-  if ( sock < 0 ) {
+  assert( !server );
+
+  setup();
+
+  prune_sockets();
+}
+
+void Connection::prune_sockets( void )
+{
+  /* don't keep old sockets if the new socket has been working for long enough */
+  if ( socks.size() > 1 ) {
+    if ( timestamp() - last_port_choice > MAX_OLD_SOCKET_AGE ) {
+      int num_to_kill = socks.size() - 1;
+      for ( int i = 0; i < num_to_kill; i++ ) {
+	socks.pop_front();
+      }
+    }
+  } else {
+    return;
+  }
+
+  /* make sure we don't have too many receive sockets open */
+  if ( socks.size() > MAX_PORTS_OPEN ) {
+    int num_to_kill = socks.size() - MAX_PORTS_OPEN;
+    for ( int i = 0; i < num_to_kill; i++ ) {
+      socks.pop_front();
+    }
+  }
+}
+
+Connection::Socket::Socket()
+  : _fd( socket( AF_INET, SOCK_DGRAM, 0 ) ),
+    _moved( false )
+{
+  if ( _fd < 0 ) {
     throw NetworkException( "socket", errno );
   }
 
@@ -110,26 +151,54 @@ void Connection::setup( void )
 #ifdef HAVE_IP_MTU_DISCOVER
   char flag = IP_PMTUDISC_DONT;
   socklen_t optlen = sizeof( flag );
-  if ( setsockopt( sock, IPPROTO_IP, IP_MTU_DISCOVER, &flag, optlen ) < 0 ) {
+  if ( setsockopt( _fd, IPPROTO_IP, IP_MTU_DISCOVER, &flag, optlen ) < 0 ) {
     throw NetworkException( "setsockopt", errno );
   }
 #endif
 
- /* set diffserv values to AF42 + ECT */
-  /*
+  /* set diffserv values to AF42 + ECT */
   uint8_t dscp = 0x92;
-  if ( setsockopt( sock, IPPROTO_IP, IP_TOS, &dscp, 1) < 0 ) {
+  if ( setsockopt( _fd, IPPROTO_IP, IP_TOS, &dscp, 1) < 0 ) {
     //    perror( "setsockopt( IP_TOS )" );
   }
-  */
+
+  /* request explicit congestion notification on received datagrams */
+#ifdef HAVE_IP_RECVTOS
+  char tosflag = true;
+  socklen_t tosoptlen = sizeof( tosflag );
+  if ( setsockopt( _fd, IPPROTO_IP, IP_RECVTOS, &tosflag, tosoptlen ) < 0 ) {
+    perror( "setsockopt( IP_RECVTOS )" );
+  }
+#endif
+}
+
+void Connection::setup( void )
+{
+  /* create socket */
+  socks.push_back( Socket() );
+
+  last_port_choice = timestamp();
+}
+
+const std::vector< int > Connection::fds( void ) const
+{
+  std::vector< int > ret;
+
+  for ( std::deque< Socket >::const_iterator it = socks.begin();
+	it != socks.end();
+	it++ ) {
+    ret.push_back( it->fd() );
+  }
+
+  return ret;
 }
 
 Connection::Connection( const char *desired_ip, const char *desired_port ) /* server */
-  : sock( -1 ),
+  : socks(),
     has_remote_addr( false ),
     remote_addr(),
     server( true ),
-    MTU( SEND_MTU ),
+    MTU( DEFAULT_SEND_MTU ),
     key(),
     session( key ),
     direction( TO_CLIENT ),
@@ -137,6 +206,9 @@ Connection::Connection( const char *desired_ip, const char *desired_port ) /* se
     saved_timestamp( -1 ),
     saved_timestamp_received_at( 0 ),
     expected_receiver_seq( 0 ),
+    last_heard( -1 ),
+    last_port_choice( -1 ),
+    last_roundtrip_success( -1 ),
     RTT_hit( false ),
     SRTT( 1000 ),
     RTTVAR( 500 ),
@@ -181,7 +253,7 @@ Connection::Connection( const char *desired_ip, const char *desired_port ) /* se
   /* try to bind to desired IP first */
   if ( desired_ip_addr != INADDR_ANY ) {
     try {
-      if ( try_bind( sock, desired_ip_addr, desired_port_no ) ) { return; }
+      if ( try_bind( sock(), desired_ip_addr, desired_port_no ) ) { return; }
     } catch ( const NetworkException& e ) {
       struct in_addr sin_addr;
       sin_addr.s_addr = desired_ip_addr;
@@ -193,7 +265,7 @@ Connection::Connection( const char *desired_ip, const char *desired_port ) /* se
 
   /* now try any local interface */
   try {
-    if ( try_bind( sock, INADDR_ANY, desired_port_no ) ) { return; }
+    if ( try_bind( sock(), INADDR_ANY, desired_port_no ) ) { return; }
   } catch ( const NetworkException& e ) {
     fprintf( stderr, "Error binding to any interface: %s: %s\n",
 	     e.function.c_str(), strerror( e.the_errno ) );
@@ -204,11 +276,11 @@ Connection::Connection( const char *desired_ip, const char *desired_port ) /* se
   throw NetworkException( "Could not bind", errno );
 }
 
-bool Connection::try_bind( int socket, uint32_t s_addr, int port )
+bool Connection::try_bind( int socket, uint32_t addr, int port )
 {
   struct sockaddr_in local_addr;
   local_addr.sin_family = AF_INET;
-  local_addr.sin_addr.s_addr = s_addr;
+  local_addr.sin_addr.s_addr = addr;
 
   int search_low = PORT_RANGE_LOW, search_high = PORT_RANGE_HIGH;
 
@@ -234,11 +306,11 @@ bool Connection::try_bind( int socket, uint32_t s_addr, int port )
 }
 
 Connection::Connection( const char *key_str, const char *ip, int port ) /* client */
-  : sock( -1 ),
+  : socks(),
     has_remote_addr( false ),
     remote_addr(),
     server( false ),
-    MTU( SEND_MTU ),
+    MTU( DEFAULT_SEND_MTU ),
     key( key_str ),
     session( key ),
     direction( TO_SERVER ),
@@ -246,6 +318,9 @@ Connection::Connection( const char *key_str, const char *ip, int port ) /* clien
     saved_timestamp( -1 ),
     saved_timestamp_received_at( 0 ),
     expected_receiver_seq( 0 ),
+    last_heard( -1 ),
+    last_port_choice( -1 ),
+    last_roundtrip_success( -1 ),
     RTT_hit( false ),
     SRTT( 1000 ),
     RTTVAR( 500 ),
@@ -269,13 +344,15 @@ Connection::Connection( const char *key_str, const char *ip, int port ) /* clien
 
 void Connection::send( string s )
 {
-  assert( has_remote_addr );
+  if ( !has_remote_addr ) {
+    return;
+  }
 
   Packet px = new_packet( s );
 
   string p = px.tostring( &session );
 
-  ssize_t bytes_sent = sendto( sock, p.data(), p.size(), 0,
+  ssize_t bytes_sent = sendto( sock(), p.data(), p.size(), MSG_DONTWAIT,
 			       (sockaddr *)&remote_addr, sizeof( remote_addr ) );
 
   if ( bytes_sent == static_cast<ssize_t>( p.size() ) ) {
@@ -286,31 +363,108 @@ void Connection::send( string s )
        flight anyway. */
     have_send_exception = true;
     send_exception = NetworkException( "sendto", errno );
+
+    if ( errno == EMSGSIZE ) {
+      MTU = 500; /* payload MTU of last resort */
+    }
+  }
+
+  uint64_t now = timestamp();
+  if ( server ) {
+    if ( now - last_heard > SERVER_ASSOCIATION_TIMEOUT ) {
+      has_remote_addr = false;
+      fprintf( stderr, "Server now detached from client.\n" );
+    }
+  } else { /* client */
+    if ( ( now - last_port_choice > PORT_HOP_INTERVAL )
+	 && ( now - last_roundtrip_success > PORT_HOP_INTERVAL ) ) {
+      hop_port();
+    }
   }
 }
 
 string Connection::recv( void )
 {
+  assert( !socks.empty() );
+  for ( std::deque< Socket >::const_iterator it = socks.begin();
+	it != socks.end();
+	it++ ) {
+    bool islast = (it + 1) == socks.end();
+    string payload;
+    try {
+      payload = recv_one( it->fd(), !islast );
+    } catch ( NetworkException & e ) {
+      if ( (e.the_errno == EAGAIN)
+	   || (e.the_errno == EWOULDBLOCK) ) {
+	assert( !islast );
+	continue;
+      } else {
+	throw e;
+      }
+    }
+
+    /* succeeded */
+    prune_sockets();
+    return payload;
+  }
+  assert( false );
+  return "";
+}
+
+string Connection::recv_one( int sock_to_recv, bool nonblocking )
+{
+  /* receive source address, ECN, and payload in msghdr structure */
   struct sockaddr_in packet_remote_addr;
+  struct msghdr header;
+  struct iovec msg_iovec;
 
-  char buf[ Session::RECEIVE_MTU ];
+  char msg_payload[ Session::RECEIVE_MTU ];
+  char msg_control[ Session::RECEIVE_MTU ];
 
-  socklen_t addrlen = sizeof( packet_remote_addr );
+  /* receive source address */
+  header.msg_name = &packet_remote_addr;
+  header.msg_namelen = sizeof( packet_remote_addr );
 
-  ssize_t received_len = recvfrom( sock, buf, Session::RECEIVE_MTU, 0, (sockaddr *)&packet_remote_addr, &addrlen );
+  /* receive payload */
+  msg_iovec.iov_base = msg_payload;
+  msg_iovec.iov_len = Session::RECEIVE_MTU;
+  header.msg_iov = &msg_iovec;
+  header.msg_iovlen = 1;
+
+  /* receive explicit congestion notification */
+  header.msg_control = msg_control;
+  header.msg_controllen = Session::RECEIVE_MTU;
+
+  /* receive flags */
+  header.msg_flags = 0;
+
+  ssize_t received_len = recvmsg( sock_to_recv, &header, nonblocking ? MSG_DONTWAIT : 0 );
 
   if ( received_len < 0 ) {
-    throw NetworkException( "recvfrom", errno );
+    throw NetworkException( "recvmsg", errno );
   }
 
-  if ( received_len > Session::RECEIVE_MTU ) {
-    char buffer[ 2048 ];
-    snprintf( buffer, 2048, "Received oversize datagram (size %d) and limit is %d\n",
-	      static_cast<int>( received_len ), Session::RECEIVE_MTU );
-    throw NetworkException( buffer, errno );
+  if ( header.msg_flags & MSG_TRUNC ) {
+    throw NetworkException( "Received oversize datagram", errno );
   }
 
-  Packet p( string( buf, received_len ), &session );
+  /* receive ECN */
+  bool congestion_experienced = false;
+
+  struct cmsghdr *ecn_hdr = CMSG_FIRSTHDR( &header );
+  if ( ecn_hdr
+       && (ecn_hdr->cmsg_level == IPPROTO_IP)
+       && (ecn_hdr->cmsg_type == IP_TOS) ) {
+    /* got one */
+    uint8_t *ecn_octet_p = (uint8_t *)CMSG_DATA( ecn_hdr );
+    assert( ecn_octet_p );
+
+    if ( (*ecn_octet_p & 0x03) == 0x03 ) {
+      congestion_experienced = true;
+    }
+  }
+
+  Packet p( string( msg_payload, received_len ), &session );
 
   dos_assert( p.direction == (server ? TO_SERVER : TO_CLIENT) ); /* prevent malicious playback to sender */
 
@@ -321,6 +475,15 @@ string Connection::recv( void )
     if ( p.timestamp != uint16_t(-1) ) {
       saved_timestamp = p.timestamp;
       saved_timestamp_received_at = timestamp();
+
+      if ( congestion_experienced ) {
+	/* signal counterparty to slow down */
+	/* this will gradually slow the counterparty down to the minimum frame rate */
+	saved_timestamp -= CONGESTION_TIMESTAMP_PENALTY;
+	if ( server ) {
+	  fprintf( stderr, "Received explicit congestion notification.\n" );
+	}
+      }
     }
 
     if ( p.timestamp_reply != uint16_t(-1) ) {
@@ -344,6 +507,7 @@ string Connection::recv( void )
 
     /* auto-adjust to remote host */
     has_remote_addr = true;
+    last_heard = timestamp();
 
     if ( server ) { /* only client can roam */
       if ( (remote_addr.sin_addr.s_addr != packet_remote_addr.sin_addr.s_addr)
@@ -364,7 +528,7 @@ int Connection::port( void ) const
   struct sockaddr_in local_addr;
   socklen_t addrlen = sizeof( local_addr );
 
-  if ( getsockname( sock, (sockaddr *)&local_addr, &addrlen ) < 0 ) {
+  if ( getsockname( sock(), (sockaddr *)&local_addr, &addrlen ) < 0 ) {
     throw NetworkException( "getsockname", errno );
   }
 
@@ -373,41 +537,7 @@ int Connection::port( void ) const
 
 uint64_t Network::timestamp( void )
 {
-#if HAVE_CLOCK_GETTIME
-  struct timespec tp;
-
-  if ( clock_gettime( CLOCK_MONOTONIC, &tp ) < 0 ) {
-    throw NetworkException( "clock_gettime", errno );
-  }
-
-  uint64_t millis = tp.tv_nsec / 1000000;
-  millis += uint64_t( tp.tv_sec ) * 1000;
-
-  return millis;
-#elif HAVE_MACH_ABSOLUTE_TIME
-  static mach_timebase_info_data_t s_timebase_info;
-
-  if (s_timebase_info.denom == 0) {
-    mach_timebase_info(&s_timebase_info);
-  }
-
-  // NB: mach_absolute_time() returns "absolute time units"
-  // We need to apply a conversion to get milliseconds.
-  return ((mach_absolute_time() * s_timebase_info.numer) / (1000000 * s_timebase_info.denom));
-#elif HAVE_GETTIMEOFDAY
-  // NOTE: If time steps backwards, timeouts may be confused.
-  struct timeval tv;
-  if ( gettimeofday(&tv, NULL) ) {
-    throw NetworkException( "gettimeofday", errno );
-  }
-
-  uint64_t millis = tv.tv_usec / 1000;
-  millis += uint64_t( tv.tv_sec ) * 1000;
-
-  return millis;
-#else
-# error "Don't know how to get a timestamp on this platform"
-#endif
+  return frozen_timestamp();
 }
 
 uint16_t Network::timestamp16( void )
@@ -443,9 +573,27 @@ uint64_t Connection::timeout( void ) const
   return RTO;
 }
 
-Connection::~Connection()
+Connection::Socket::~Socket()
 {
-  if ( close( sock ) < 0 ) {
-    throw NetworkException( "close", errno );
+  if ( !_moved ) {
+    if ( close( _fd ) < 0 ) {
+      throw NetworkException( "close", errno );
+    }
   }
 }
+
+Connection::Socket::Socket( const Socket & other )
+  : _fd( other._fd ),
+    _moved( false )
+{
+  other.move();
+}
+
+const Connection::Socket & Connection::Socket::operator=( const Socket & other )
+{
+  _fd = other._fd;
+  
+  other.move();
+
+  return *this;
+}
