@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #include <stdio.h>
@@ -57,7 +71,8 @@ std::string Display::new_frame( bool initialized, const Framebuffer &last, const
 	snprintf( tmp, 64, "%lc", *i );
 	frame.append( tmp );
       }
-      frame.append( "\033\\" );
+      frame.append( "\007" );
+      /* ST is more correct, but BEL more widely supported */
     } else {
       /* write Icon Name */
       frame.append( "\033]1;" );
@@ -68,7 +83,7 @@ std::string Display::new_frame( bool initialized, const Framebuffer &last, const
 	snprintf( tmp, 64, "%lc", *i );
 	frame.append( tmp );
       }
-      frame.append( "\033\\" );
+      frame.append( "\007" );
 
       frame.append( "\033]2;" );
       const std::deque<wchar_t> &window_title( f.get_window_title() );
@@ -78,7 +93,7 @@ std::string Display::new_frame( bool initialized, const Framebuffer &last, const
 	snprintf( tmp, 64, "%lc", *i );
 	frame.append( tmp );
       }
-      frame.append( "\033\\" );
+      frame.append( "\007" );
     }
 
   }
