@@ -14,6 +14,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+    In addition, as a special exception, the copyright holders give
+    permission to link the code of portions of this program with the
+    OpenSSL library under certain conditions as described in each
+    individual source file, and distribute linked combinations including
+    the two.
+
+    You must obey the GNU General Public License in all respects for all
+    of the code used other than OpenSSL. If you modify file(s) with this
+    exception, you may extend this exception to your version of the
+    file(s), but you are not obligated to do so. If you do not wish to do
+    so, delete this exception statement from your version. If you delete
+    this exception statement from all source files in the program, then
+    also delete it here.
 */
 
 #ifndef FATAL_ASSERT_HPP
@@ -26,12 +40,12 @@ static void fatal_error( const char *expression, const char *file, int line, con
 {
   fprintf( stderr, "Fatal assertion failure in function %s at %s:%d\nFailed test: %s\n",
            function, file, line, expression );
-  exit( 2 );
+  abort();
 }
 
 #define fatal_assert(expr)						\
   ((expr)								\
    ? (void)0								\
-   : fatal_error (__STRING(expr), __FILE__, __LINE__, __PRETTY_FUNCTION__ ))
+   : fatal_error (#expr, __FILE__, __LINE__, __PRETTY_FUNCTION__ ))
 
 #endif
